@@ -65,6 +65,7 @@ struct _BraseroDrivePrivate
 {
 	BraseroMedium *medium;
 	BraseroDriveCaps caps;
+	gchar *name;
 	gchar *path;
 	gchar *block_path;
 	gchar *udi;
@@ -370,11 +371,7 @@ brasero_drive_can_use_exclusively (BraseroDrive *drive)
 	g_return_val_if_fail (drive != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
-#if defined(HAVE_STRUCT_USCSI_CMD)
-	device = brasero_drive_get_block_device (drive);
-#else
 	device = brasero_drive_get_device (drive);
-#endif
 
 	handle = brasero_device_handle_open (device, TRUE, NULL);
 	if (!handle)
@@ -494,8 +491,6 @@ gchar *
 brasero_drive_get_display_name (BraseroDrive *drive)
 {
 	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	LibHalContext *ctx;
 
 	g_return_val_if_fail (drive != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
@@ -508,12 +503,7 @@ brasero_drive_get_display_name (BraseroDrive *drive)
 		return g_strdup (_("Image File"));
 	}
 
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
-	return libhal_device_get_property_string (ctx,
-						  priv->udi,
-	  					  DEVICE_MODEL,
-						  NULL);
+	return g_strdup (priv->name);
 }
 
 /**
@@ -670,6 +660,12 @@ brasero_drive_finalize (GObject *object)
 	BraseroDrivePrivate *priv;
 
 	priv = BRASERO_DRIVE_PRIVATE (object);
+
+	if (priv->name) {
+		libhal_free_string (priv->name);
+		priv->name = NULL;
+	}
+
 	if (priv->path) {
 		libhal_free_string (priv->path);
 		priv->path = NULL;
@@ -915,6 +911,11 @@ brasero_drive_init_real (BraseroDrive *drive)
 	watch = brasero_hal_watch_get_default ();
 	ctx = brasero_hal_watch_get_ctx (watch);
 
+	priv->name = libhal_device_get_property_string (ctx,
+							priv->udi,
+							DEVICE_MODEL,
+							NULL);
+
 	priv->path = libhal_device_get_property_string (ctx,
 							priv->udi,
 							BLOCK_DEVICE,
@@ -949,6 +950,12 @@ brasero_drive_init_real (BraseroDrive *drive)
 		priv->caps |= BRASERO_DRIVE_CAPS_DVDR_PLUS_DL;
 	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdplusrwdl", NULL))
 		priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL;
+	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdram", NULL))
+		priv->caps |= BRASERO_DRIVE_CAPS_DVDRAM;
+	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.bdr", NULL))
+		priv->caps |= BRASERO_DRIVE_CAPS_BDR;
+	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.bdre", NULL))
+		priv->caps |= BRASERO_DRIVE_CAPS_BDRW;
 
 	BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
 
