@@ -27,11 +27,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libhal.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <libnautilus-extension/nautilus-location-widget-provider.h>
 
+#include "brasero-media.h"
+#include "brasero-medium-monitor.h"
+#include "brasero-drive.h"
+#include "brasero-medium.h"
 #include "nautilus-burn-bar.h"
 
 #define BURN_URI "burn:///"
@@ -71,58 +75,31 @@ static void  nautilus_disc_burn_register_type (GTypeModule *module);
 
 static GObjectClass *parent_class;
 
-#undef DEBUG_ENABLE
-
-#ifdef DEBUG_ENABLE
-#ifdef G_HAVE_ISO_VARARGS
-#  define DEBUG_PRINT(...) debug_print (__VA_ARGS__);
-#elif defined(G_HAVE_GNUC_VARARGS)
-#  define DEBUG_PRINT(args...) debug_print (args);
-#endif
-#else
-#ifdef G_HAVE_ISO_VARARGS
-#  define DEBUG_PRINT(...)
-#elif defined(G_HAVE_GNUC_VARARGS)
-#  define DEBUG_PRINT(args...)
-#endif
-#endif
-
-#ifdef DEBUG_ENABLE
-static FILE *debug_out = NULL;
+//#define DEBUG_PRINT(format_MACRO,...)           g_print (format_MACRO, ##__VA_ARGS__);
+#define DEBUG_PRINT(format_MACRO,...)             
 
 static void
-debug_init (void)
-{
-        const char path [50] = "burn_extension_debug_XXXXXX";
-        int  fd = g_file_open_tmp (path, NULL, NULL);
-        if (fd >= 0) {
-                debug_out = fdopen (fd, "a");
-        }
-}
-
-static void
-debug_print (const char *format, ...)
-{
-        va_list args;
-
-        va_start (args, format);
-        vfprintf ((debug_out ? debug_out : stderr), format, args);
-        vfprintf (stderr, format, args);
-        va_end (args);
-        if (debug_out)
-                fflush (debug_out);
-}
-#endif
-
-static void
-launch_process (char **argv, GtkWindow *parent)
+launch_process (GPtrArray *argv, GtkWindow *parent)
 {
         GtkWidget *dialog;
         GError *error;
+        gchar **args;
+
+        if (parent && GTK_WIDGET (parent)->window) {
+                guint xid;
+
+		xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (GTK_WIDGET (parent)->window));
+                if (xid > 0) {
+                        g_ptr_array_add (argv, g_strdup ("-x"));
+                        g_ptr_array_add (argv, g_strdup_printf ("%d", xid));
+                }
+        }
 
         error = NULL;
+        g_ptr_array_add (argv, NULL);
+        args = (gchar **) g_ptr_array_free (argv, FALSE);
         if (!g_spawn_async (NULL,
-                            argv, NULL,
+                            args, NULL,
                             0,
                             NULL, NULL,
                             NULL,
@@ -144,21 +121,21 @@ launch_process (char **argv, GtkWindow *parent)
 
                 g_error_free (error);
         }
+
+        g_strfreev (args);
 }
 
 static void
 launch_brasero_on_window (GtkWindow *window)
 {
-        char *argv [3];
+        int i;
+        GPtrArray        *argv = NULL;
 
-        argv [0] = g_build_filename (BINDIR, "brasero", NULL);
-        argv [1] = g_strdup ("-n");
-        argv [2] = NULL;
+        argv = g_ptr_array_new ();
+        g_ptr_array_add (argv, g_build_filename (BINDIR, "brasero", NULL));
+        g_ptr_array_add (argv, g_strdup ("-n"));
 
         launch_process (argv, window);
-
-        g_free (argv [0]);
-        g_free (argv [1]);
 }
 
 static void
@@ -168,33 +145,22 @@ write_activate_cb (NautilusMenuItem *item,
         launch_brasero_on_window (GTK_WINDOW (user_data));
 }
 
-static char *
-uri_to_path (const char *uri)
-{
-        GFile *file;
-        char  *path;
-
-        file = g_file_new_for_uri (uri);
-        path = g_file_get_path (file);
-        g_object_unref (file);
-        return path;
-}
-
 static void
 write_iso_activate_cb (NautilusMenuItem *item,
                        gpointer          user_data)
 {
         NautilusFileInfo *file_info;
-        char             *argv [4];
+        GPtrArray        *argv = NULL;
         char             *uri;
-        char             *image_name;
+        char             *image_path;
+        int               i;
 
         file_info = g_object_get_data (G_OBJECT (item), "file_info");
 
         uri = nautilus_file_info_get_uri (file_info);
-        image_name = uri_to_path (uri);
+        image_path = g_filename_from_uri (uri, NULL, NULL);
 
-        if (image_name == NULL) {
+        if (image_path == NULL) {
                 g_warning ("Can not get local path for URI %s", uri);
                 g_free (uri);
                 return;
@@ -202,23 +168,20 @@ write_iso_activate_cb (NautilusMenuItem *item,
 
         g_free (uri);
 
-        argv [0] = g_build_filename (BINDIR, "brasero", NULL);
-        argv [1] = g_strdup ("-i");
-        argv [2] = image_name;
-        argv [3] = NULL;
+        argv = g_ptr_array_new ();
+        g_ptr_array_add (argv, g_build_filename (BINDIR, "brasero", NULL));
+        g_ptr_array_add (argv, g_strdup ("-i"));
+        g_ptr_array_add (argv, image_path);
 
         launch_process (argv, GTK_WINDOW (user_data));
-
-        g_free (argv [1]);
-        g_free (argv [0]);
-        g_free (image_name);
 }
 
 static void
 copy_disc_activate_cb (NautilusMenuItem *item,
                        gpointer          user_data)
 {
-        char             *argv [4];
+        int               i;
+        GPtrArray        *argv = NULL;
         char             *device_path;
 
         device_path = g_object_get_data (G_OBJECT (item), "drive_device_path");
@@ -228,22 +191,20 @@ copy_disc_activate_cb (NautilusMenuItem *item,
                 return;
         }
 
-        argv [0] = g_build_filename (BINDIR, "brasero", NULL);
-        argv [1] = g_strdup ("-c");
-        argv [2] = device_path;
-        argv [3] = NULL;
+        argv = g_ptr_array_new ();
+        g_ptr_array_add (argv, g_build_filename (BINDIR, "brasero", NULL));
+        g_ptr_array_add (argv, g_strdup ("-c"));
+        g_ptr_array_add (argv, g_strdup (device_path));
 
         launch_process (argv, GTK_WINDOW (user_data));
-
-        g_free (argv [1]);
-        g_free (argv [0]);
 }
 
 static void
 blank_disc_activate_cb (NautilusMenuItem *item,
                         gpointer          user_data)
 {
-        char             *argv [4];
+        int               i;
+        GPtrArray        *argv = NULL;
         char             *device_path;
 
         device_path = g_object_get_data (G_OBJECT (item), "drive_device_path");
@@ -253,22 +214,20 @@ blank_disc_activate_cb (NautilusMenuItem *item,
                 return;
         }
 
-        argv [0] = g_build_filename (BINDIR, "brasero", NULL);
-        argv [1] = g_strdup ("-b");
-        argv [2] = device_path;
-        argv [3] = NULL;
+        argv = g_ptr_array_new ();
+        g_ptr_array_add (argv, g_build_filename (BINDIR, "brasero", NULL));
+        g_ptr_array_add (argv, g_strdup ("-b"));
+        g_ptr_array_add (argv, g_strdup (device_path));
 
         launch_process (argv, GTK_WINDOW (user_data));
-
-        g_free (argv [1]);
-        g_free (argv [0]);
 }
 
 static void
 check_disc_activate_cb (NautilusMenuItem *item,
                         gpointer          user_data)
 {
-        char             *argv [4];
+        int               i;
+        GPtrArray        *argv = NULL;
         char             *device_path;
 
         device_path = g_object_get_data (G_OBJECT (item), "drive_device_path");
@@ -278,81 +237,43 @@ check_disc_activate_cb (NautilusMenuItem *item,
                 return;
         }
 
-        argv [0] = g_build_filename (BINDIR, "brasero", NULL);
-        argv [1] = g_strdup ("-k");
-        argv [2] = device_path;
-        argv [3] = NULL;
+        argv = g_ptr_array_new ();
+        g_ptr_array_add (argv, g_build_filename (BINDIR, "brasero", NULL));
+        g_ptr_array_add (argv, g_strdup ("-k"));
+        g_ptr_array_add (argv, g_strdup (device_path));
 
         launch_process (argv, GTK_WINDOW (user_data));
-
-        g_free (argv [1]);
-        g_free (argv [0]);
-}
-
-static LibHalContext *
-get_hal_context (void)
-{
-        static LibHalContext *ctx = NULL;
-        DBusError             error;
-        DBusConnection       *dbus_conn;
-
-        if (ctx == NULL) {
-                ctx = libhal_ctx_new ();
-                if (ctx == NULL) {
-                        g_warning ("Could not create a HAL context");
-                } else {
-                        dbus_error_init (&error);
-                        dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-
-                        if (dbus_error_is_set (&error)) {
-                                g_warning ("Could not connect to system bus: %s", error.message);
-                                dbus_error_free (&error);
-                                return NULL;
-                        }
-
-                        libhal_ctx_set_dbus_connection (ctx, dbus_conn);
-
-                        if (! libhal_ctx_init (ctx, &error)) {
-                                g_warning ("Could not initalize the HAL context: %s",
-                                           error.message);
-
-                                if (dbus_error_is_set (&error))
-                                        dbus_error_free (&error);
-
-                                libhal_ctx_free (ctx);
-                                ctx = NULL;
-                        }
-                }
-        }
-
-        return ctx;
 }
 
 static gboolean
 volume_is_blank (GVolume *volume)
 {
-        LibHalContext *ctx;
-        char          *udi;
-        gboolean       is_blank;
-
-        ctx = get_hal_context ();
-        if (ctx == NULL) {
-                return FALSE;
-        }
+        BraseroMediumMonitor *monitor;
+        BraseroMedium        *medium;
+        BraseroDrive         *drive;
+        gchar                *device;
+        gboolean              is_blank;
 
         is_blank = FALSE;
 
-        udi = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-        DEBUG_PRINT ("Got udi: %s\n", udi);
-        if (udi != NULL) {
-                is_blank = libhal_device_get_property_bool (ctx,
-                                                            udi,
-                                                            "volume.disc.is_blank",
-                                                            NULL);
-        }
-        g_free (udi);
+        device = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+        if (!device)
+                return FALSE;
+
+        DEBUG_PRINT ("Got device: %s\n", device);
+
+        monitor = brasero_medium_monitor_get_default ();
+        drive = brasero_medium_monitor_get_drive (monitor, device);
+        g_object_unref (monitor);
+        g_free (device);
+
+        if (drive == NULL)
+                return FALSE;
+
+        medium = brasero_drive_get_medium (drive);
+        is_blank = (brasero_medium_get_status (medium) & BRASERO_MEDIUM_BLANK);
+        g_object_unref (drive);
 
-        DEBUG_PRINT ("is-blank: %d\n", is_blank);
         return is_blank;
 }
 
@@ -377,39 +298,28 @@ drive_get_first_volume (GDrive *drive)
 }
 
 static gboolean
-drive_is_cd_device (GDrive *drive)
+drive_is_cd_device (GDrive *gdrive)
 {
-        LibHalContext *ctx;
-        char          *udi;
-        gboolean       is_cd;
+        BraseroMediumMonitor *monitor;
+        BraseroDrive         *drive;
+        gchar                *device;
 
-        ctx = get_hal_context ();
-        if (ctx == NULL) {
+        device = g_drive_get_identifier (gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+        if (!device)
                 return FALSE;
-        }
 
-        is_cd = FALSE;
+        DEBUG_PRINT ("Got device: %s\n", device);
 
-        udi = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-        DEBUG_PRINT ("Got udi: %s\n", udi);
+        monitor = brasero_medium_monitor_get_default ();
+        drive = brasero_medium_monitor_get_drive (monitor, device);
+        g_object_unref (monitor);
+        g_free (device);
 
-        if (udi != NULL) {
-                char *drive_type;
-                drive_type = libhal_device_get_property_string (ctx,
-                                                                udi,
-                                                                "storage.drive_type",
-                                                                NULL);
-                DEBUG_PRINT ("Got drive type: '%s'\n", drive_type);
-                if (drive_type != NULL && strcmp (drive_type, "cdrom") == 0) {
-                        is_cd = TRUE;
-                }
-                g_free (drive_type);
-        }
-        g_free (udi);
-
-        DEBUG_PRINT ("is-cdrom: %d\n", is_cd);
-
-        return is_cd;
+        if (drive == NULL)
+                return FALSE;
+        
+        g_object_unref (drive);
+        return TRUE;
 }
 
 static GList *
@@ -443,13 +353,13 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
 
         if (file == NULL) {
                 DEBUG_PRINT ("No file found\n");
-
                 return NULL;
         }
 
         mime_type = nautilus_file_info_get_mime_type (file_info);
         DEBUG_PRINT ("Mime type: %s\n", mime_type);
         if (! mime_type) {
+                g_object_unref (file);
                 return NULL;
         }
 
@@ -468,7 +378,7 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
                 g_object_set_data (G_OBJECT (item), "file_info", file_info);
                 g_object_set_data (G_OBJECT (item), "window", window);
                 g_signal_connect (item, "activate",
-                                  G_CALLBACK (write_iso_activate_cb), NULL);
+                                  G_CALLBACK (write_iso_activate_cb), window);
                 items = g_list_append (items, item);
         }
 
@@ -522,7 +432,7 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
                 g_object_set_data (G_OBJECT (item), "window", window);
                 g_object_set_data_full (G_OBJECT (item), "drive_device_path", g_strdup (device_path), g_free);
                 g_signal_connect (item, "activate",
-                                  G_CALLBACK (copy_disc_activate_cb), NULL);
+                                  G_CALLBACK (copy_disc_activate_cb), window);
                 items = g_list_append (items, item);
 
                 /* ... or if it's a rewritable medium to blank it ... */
@@ -534,7 +444,7 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
                 g_object_set_data (G_OBJECT (item), "window", window);
                 g_object_set_data_full (G_OBJECT (item), "drive_device_path", g_strdup (device_path), g_free);
                 g_signal_connect (item, "activate",
-                                  G_CALLBACK (blank_disc_activate_cb), NULL);
+                                  G_CALLBACK (blank_disc_activate_cb), window);
                 items = g_list_append (items, item);
 
                 /* ... or verify medium. */
@@ -546,12 +456,14 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
                 g_object_set_data (G_OBJECT (item), "window", window);
                 g_object_set_data_full (G_OBJECT (item), "drive_device_path", g_strdup (device_path), g_free);
                 g_signal_connect (item, "activate",
-                                  G_CALLBACK (check_disc_activate_cb), NULL);
+                                  G_CALLBACK (check_disc_activate_cb), window);
                 items = g_list_append (items, item);
 
                 g_free (device_path);
         }
 
+        g_object_unref (file);
+
         if (drive != NULL) {
                 g_object_unref (drive);
         }
@@ -561,6 +473,7 @@ nautilus_disc_burn_get_file_items (NautilusMenuProvider *provider,
 
         g_free (mime_type);
 
+        DEBUG_PRINT ("Items returned\n");
         return items;
 }
 
@@ -584,7 +497,8 @@ nautilus_disc_burn_get_background_items (NautilusMenuProvider *provider,
                                                _("Write contents to a CD or DVD disc"),
                                                "brasero");
                 g_signal_connect (item, "activate",
-                                  G_CALLBACK (write_activate_cb), window);
+                                  G_CALLBACK (write_activate_cb),
+                                  window);
                 items = g_list_append (items, item);
 
                 g_object_set (item, "sensitive", ! NAUTILUS_DISC_BURN (provider)->priv->empty, NULL);
@@ -827,11 +741,6 @@ static void
 nautilus_disc_burn_instance_init (NautilusDiscBurn *burn)
 {
         burn->priv = NAUTILUS_DISC_BURN_GET_PRIVATE (burn);
-
-#ifdef DEBUG_ENABLE
-        debug_init ();
-#endif
-
         burn->priv->start_monitor_id = g_timeout_add_seconds (1,
                                                               (GSourceFunc)start_monitor,
                                                               burn);
@@ -934,25 +843,21 @@ nautilus_disc_burn_register_type (GTypeModule *module)
 void
 nautilus_module_initialize (GTypeModule *module)
 {
-        nautilus_disc_burn_register_type (module);
-        bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+        DEBUG_PRINT ("Initializing nautilus disc recorder\n");
 
-	/* As long as it's not a library I don't see how to do it differently */
-	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   "/usr/share/brasero/icons");
-	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   "/usr/local/share/brasero/icons");
+        brasero_media_library_start ();
+        DEBUG_PRINT ("Libbrasero-media started\n");
+
+        nautilus_disc_burn_register_type (module);
 }
 
 void
 nautilus_module_shutdown (void)
 {
-        LibHalContext *ctx;
-        ctx = get_hal_context ();
-        if (ctx != NULL) {
-                libhal_ctx_free (ctx);
-        }
+        DEBUG_PRINT ("Shutting down nautilus disc recorder\n");
+
+        /* Don't do that in case another module would need the library */
+        //brasero_media_library_stop ();
 }
 
 void
@@ -961,7 +866,7 @@ nautilus_module_list_types (const GType **types,
 {
         static GType type_list [1];
 
-        type_list[0] = NAUTILUS_TYPE_DISC_BURN;
+        type_list [0] = NAUTILUS_TYPE_DISC_BURN;
 
         *types = type_list;
         *num_types = 1;
