@@ -29,26 +29,7 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include <gtk/gtkstock.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkbin.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkexpander.h>
-#include <gtk/gtksizegroup.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtk.h>
 
 #include "brasero-disc-message.h"
 
@@ -289,7 +270,7 @@ brasero_disc_message_add_button (BraseroDiscMessage *self,
 
 	priv = BRASERO_DISC_MESSAGE_PRIVATE (self);
 
-	button = gtk_button_new_with_mnemonic (text);
+	button = gtk_button_new_from_stock (text);
 
 	/* only add buttons to group if the text is not wrapped. Otherwise
 	 * buttons would be too big. */
