@@ -31,7 +31,7 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
-#include <gdk/gdkpixbuf.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include <gst/gst.h>
 
@@ -39,7 +39,6 @@ G_BEGIN_DECLS
 
 typedef enum {
 	BRASERO_METADATA_FLAG_NONE			= 0,
-	BRASERO_METADATA_FLAG_FAST			= 1,
 	BRASERO_METADATA_FLAG_SILENCES			= 1 << 1,
 	BRASERO_METADATA_FLAG_MISSING			= 1 << 2,
 	BRASERO_METADATA_FLAG_THUMBNAIL			= 1 << 3
