@@ -31,11 +31,14 @@
 
 #include <gconf/gconf-client.h>
 
+#include "brasero-media-private.h"
+
 #include "burn-basics.h"
 #include "burn-debug.h"
 #include "burn-plugin.h"
 #include "burn-plugin-private.h"
 #include "burn-caps.h"
+#include "brasero-media.h"
 
 #define BRASERO_PLUGIN_PRIORITY_KEY			"/apps/brasero/config/priority"
 
@@ -98,6 +101,8 @@ struct _BraseroPluginPrivate
 	GSList *blank_flags;
 
 	BraseroPluginProcessFlag process_flags;
+
+	guint compulsory:1;
 };
 
 static const gchar *default_icon = "gtk-cdrom";
@@ -122,6 +127,25 @@ enum
 static GTypeModuleClass* parent_class = NULL;
 static guint plugin_signals [LAST_SIGNAL] = { 0 };
 
+void
+brasero_plugin_set_compulsory (BraseroPlugin *self,
+			       gboolean compulsory)
+{
+	BraseroPluginPrivate *priv;
+
+	priv = BRASERO_PLUGIN_PRIVATE (self);
+	priv->compulsory = compulsory;
+}
+
+gboolean
+brasero_plugin_get_compulsory (BraseroPlugin *self)
+{
+	BraseroPluginPrivate *priv;
+
+	priv = BRASERO_PLUGIN_PRIVATE (self);
+	return priv->compulsory;
+}
+
 void
 brasero_plugin_set_active (BraseroPlugin *self, gboolean active)
 {
@@ -387,20 +411,18 @@ brasero_plugin_get_gconf_priority_key (BraseroPlugin *self)
 	caps = brasero_burn_caps_get_default ();
 
 	/* make sure the name used has characters usable by GConf */
-	if (brasero_burn_caps_plugin_can_burn (caps, self) == BRASERO_BURN_OK)
+	if (brasero_plugin_can_burn (self) == BRASERO_BURN_OK)
 		type = 0;
-	else if (brasero_burn_caps_plugin_can_image (caps, self) == BRASERO_BURN_OK)
+	else if (brasero_plugin_can_image (self) == BRASERO_BURN_OK)
 		type = 1;
-	else if (brasero_burn_caps_plugin_can_convert (caps, self) == BRASERO_BURN_OK)
+	else if (brasero_plugin_can_convert (self) == BRASERO_BURN_OK)
 		type = 2;
 
-	gconf_name = g_strdup (priv->name);
-	g_strdelimit (gconf_name, " +()", '_');
+	gconf_name = gconf_escape_key (priv->name, -1);
 	priority_path = g_strdup_printf ("%s/%s-%s",
 					 BRASERO_PLUGIN_PRIORITY_KEY,
 					 gconf_name,
 					 category [type]);
-
 	g_free (gconf_name);
 	g_object_unref (caps);
 
@@ -427,19 +449,13 @@ brasero_plugin_define (BraseroPlugin *self,
 }
 
 void
-brasero_plugin_register_group (BraseroPlugin *self,
-			       const gchar *name)
+brasero_plugin_set_group (BraseroPlugin *self,
+			  gint group_id)
 {
-	guint id;
-	BraseroBurnCaps *caps;
 	BraseroPluginPrivate *priv;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-
-	caps = brasero_burn_caps_get_default ();
-	id = brasero_burn_caps_register_plugin_group (caps, name);
-	priv->group = id;
-	g_object_unref (caps);
+	priv->group = group_id;
 }
 
 guint
@@ -492,6 +508,14 @@ brasero_plugin_set_flags_real (GSList *flags_list,
 		flags->media = media;
 		flags_list = g_slist_prepend (flags_list, flags);
 	}
+	else for (pair = flags->pairs; pair; pair = pair->next) {
+		/* have a look at the BraseroPluginFlagPair to see if there
+		 * is an exactly similar pair of flags or at least which
+		 * encompasses it to avoid redundancy. */
+		if ((pair->supported & supported) == supported
+		&&  (pair->compulsory & compulsory) == compulsory)
+			return flags_list;
+	}
 
 	pair = g_new0 (BraseroPluginFlagPair, 1);
 	pair->supported = supported;
@@ -510,12 +534,22 @@ brasero_plugin_set_flags (BraseroPlugin *self,
 			  BraseroBurnFlag compulsory)
 {
 	BraseroPluginPrivate *priv;
+	GSList *list;
+	GSList *iter;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	priv->flags = brasero_plugin_set_flags_real (priv->flags,
-						     media,
-						     supported,
-						     compulsory);
+
+	list = brasero_media_get_all_list (media);
+	for (iter = list; iter; iter = iter->next) {
+		BraseroMedia medium;
+
+		medium = GPOINTER_TO_INT (iter->data);
+		priv->flags = brasero_plugin_set_flags_real (priv->flags,
+							     medium,
+							     supported,
+							     compulsory);
+	}
+	g_slist_free (list);
 }
 
 static gboolean
@@ -535,6 +569,7 @@ brasero_plugin_get_all_flags (GSList *flags_list,
 
 	flags = brasero_plugin_get_flags (flags_list, media);
 	if (!flags) {
+
 		if (supported_retval)
 			*supported_retval = BRASERO_BURN_FLAG_NONE;
 		if (compulsory_retval)
@@ -944,13 +979,14 @@ brasero_plugin_priority_changed (GConfClient *client,
 	priv = BRASERO_PLUGIN_PRIVATE (self);
 
 	value = gconf_entry_get_value (entry);
-	if (value->type != GCONF_VALUE_INT)
+	if (!value)
+		priv->priority = 0;
+	else if (value->type != GCONF_VALUE_INT)
 		return;
-
-	self = BRASERO_PLUGIN (data);
+	else
+		priv->priority = gconf_value_get_int (value);
 
 	is_active = brasero_plugin_get_active (self);
-	priv->priority = gconf_value_get_int (value);
 
 	g_object_notify (G_OBJECT (self), "priority");
 	if (is_active != brasero_plugin_get_active (self))
@@ -1145,6 +1181,7 @@ brasero_plugin_init (BraseroPlugin *object)
 
 	priv = BRASERO_PLUGIN_PRIVATE (object);
 	priv->type = G_TYPE_NONE;
+	priv->compulsory = TRUE;
 }
 
 static void
