@@ -43,7 +43,7 @@
 #include "burn-job.h"
 #include "burn-plugin.h"
 #include "burn-volume.h"
-#include "burn-drive.h"
+#include "brasero-drive.h"
 
 BRASERO_PLUGIN_BOILERPLATE (BraseroReadom, brasero_readom, BRASERO_TYPE_PROCESS, BraseroProcess);
 static GObjectClass *parent_class = NULL;
@@ -398,7 +398,7 @@ brasero_readom_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	brasero_plugin_define (plugin,
 			       "readom",
-			       _("Use readom to image CDs and DVDs"),
+			       _("Use readom to create disc images"),
 			       "Philippe Rouquier",
 			       1);
 
