@@ -838,6 +838,7 @@ static BraseroBurnResult
 brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 {
 	BraseroLibisofsPrivate *priv;
+	GError *thread_error = NULL;
 
 	priv = BRASERO_LIBISOFS_PRIVATE (self);
 	if (priv->thread)
@@ -856,10 +857,16 @@ brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 	priv->thread = g_thread_create (brasero_libisofs_create_volume_thread,
 					self,
 					TRUE,
-					error);
+					&thread_error);
 	g_mutex_unlock (priv->mutex);
-	if (!priv->thread)
+
+	/* Reminder: this is not necessarily an error as the thread may have finished */
+	//if (!priv->thread)
+	//	return BRASERO_BURN_ERR;
+	if (thread_error) {
+		g_propagate_error (error, thread_error);
 		return BRASERO_BURN_ERR;
+	}
 
 	return BRASERO_BURN_OK;
 }
